#!/usr/bin/env python3

import typer
from pantam_cli.action import run_action
from pantam_cli.init import run_init
from pantam_cli.serve import run_serve


run = typer.Typer()


@run.command()
def init():
    """Configure a Pantam application"""
    run_init()


@run.command()
def action(file: str):
    """Create a new action route"""
    run_action(file)


@run.command()
def serve(dev: bool = False):
    """Create a new action route"""
    run_serve(dev)


if __name__ == "__main__":
    run()



from typing import TypedDict, Union
from os import getcwd, mkdir
from json import dumps, loads
from pathlib import Path
import sys
from pantam_cli.utils.messages import error_msg, write_error


def create_file(file_path: Union[str, Path], data: str) -> None:
    """Create a file"""
    new_file = open(file_path, "w")
    new_file.write(data)
    new_file.close()


def create_folder(folder_path: str) -> None:
    """Create a folder"""
    mkdir(folder_path)


def make_class_name(file_name: str) -> str:
    """Format file name to class name"""
    return file_name.replace(r".py", "").replace("_", " ").title().replace(" ", "")


class CliOptions(TypedDict):
    actions_folder: str
    entrypoint: str
    dev_port: int
    port: int


def create_pantamrc_file(options: CliOptions) -> None:
    """Create pantamrc.json file"""
    create_file("./.pantamrc.json", dumps(options))


def load_pantamrc_file() -> CliOptions:
    """Load pantamrc.json file"""
    cwd = Path(getcwd())
    try:
        config = open(cwd / ".pantamrc.json", "r")
        options: CliOptions = loads(config.read())
        return options
    except:
        write_error(
            error_msg("Cannot find .pantamrc.json file. Trying running `pantam init`")
        )
        sys.exit(1)



#!/usr/bin/env python3

from os import getcwd
from pathlib import Path
import sys
from pantam_cli.utils.filesystem import (
    create_file,
    make_class_name,
    load_pantamrc_file,
)
from pantam_cli.utils.messages import (
    info_msg,
    error_msg,
    success_msg,
    write_error,
    write_msg,
    NewLine,
)
from pantam_cli.utils.templates import action_template
from pantam_cli.utils import clear


def action(action_file: str) -> None:
    """Create action file"""
    clear()

    options = load_pantamrc_file()

    try:
        actions_folder = options["actions_folder"]
        write_msg(info_msg("Creating %s/%s file..." % (actions_folder, action_file)))
        cwd = Path(getcwd())
        create_file(
            cwd / actions_folder / action_file,
            action_template(make_class_name(action_file)),
        )
        write_msg(
            success_msg(" Done!"), NewLine.after,
        )
    except FileExistsError:
        write_msg(
            error_msg(" file exists, skipping"), NewLine.after,
        )


def run_action(action_file: str) -> None:
    """CLI runner for action()"""
    try:
        action(action_file)
        write_msg(success_msg("Your new action `%s` is ready!" % action_file))
    except Exception as error:
        write_error(error_msg(str(error)))


if __name__ == "__main__":
    file_name = sys.argv[1]
    run_action(file_name)



#!/usr/bin/env python3

from typing import List
from os import getcwd, path
from pathlib import Path
from PyInquirer import prompt
from pantam_cli.utils.filesystem import (
    create_file,
    create_folder,
    make_class_name,
    create_pantamrc_file,
)
from pantam_cli.utils.messages import (
    info_msg,
    error_msg,
    success_msg,
    write_error,
    write_msg,
    NewLine,
    welcome_msg,
    name_index_file_msg,
    name_actions_folder_msg,
    name_actions_file_msg,
    create_actions_file_msg,
    confirm_structure_msg,
)
from pantam_cli.utils.templates import action_template, index_template
from pantam_cli.utils.errors import CancelError
from pantam_cli.utils import clear


def init() -> None:
    """Setup Pantam project"""
    clear()

    write_msg(welcome_msg())

    folder_name = path.basename(getcwd())

    answers = prompt(
        [
            {
                "type": "input",
                "name": "index_file",
                "message": name_index_file_msg(),
                "default": "%s.py" % folder_name,
            },
            {
                "type": "input",
                "name": "actions_folder",
                "message": name_actions_folder_msg(),
                "default": "actions",
            },
            {
                "type": "confirm",
                "name": "create_actions_file",
                "message": create_actions_file_msg(False),
                "default": True,
            },
        ]
    )

    index_file = answers["index_file"]
    actions_folder = answers["actions_folder"]
    do_create_action_files = answers["create_actions_file"]

    action_files: List[str] = []

    while do_create_action_files:
        answers = prompt(
            [
                {
                    "type": "input",
                    "name": "actions_file",
                    "message": name_actions_file_msg(),
                    "default": "index.py" if len(action_files) == 0 else "",
                },
                {
                    "type": "confirm",
                    "name": "create_actions_file",
                    "message": create_actions_file_msg(True),
                    "default": False,
                },
            ]
        )
        if answers["actions_file"] not in action_files:
            action_files.append(answers["actions_file"])
        do_create_action_files = answers["create_actions_file"]

    action_files_map = list(map(lambda file_name: "|  |  %s" % file_name, action_files))
    action_files_flat = "\n".join(action_files_map)

    structure = """
| {index_file}
| {actions_folder}
{actions_files}""".format(
        index_file=index_file,
        actions_folder=actions_folder,
        actions_files=action_files_flat,
    )

    answers = prompt(
        [
            {
                "type": "confirm",
                "name": "confirm",
                "message": confirm_structure_msg(structure),
                "default": True,
            },
        ]
    )

    if not answers["confirm"]:
        raise CancelError("Cancelled!")

    write_msg(
        info_msg("Building your app! 🚀"), NewLine.both,
    )

    try:
        write_msg(info_msg("Creating %s file..." % index_file),)
        create_file(index_file, index_template())
        create_file("__init__.py", "")
        write_msg(success_msg(" done!"), NewLine.after)
    except FileExistsError:
        write_msg(error_msg(" file exists, skipping"), NewLine.after)

    try:
        write_msg(info_msg("Creating %s folder..." % actions_folder))
        create_folder(actions_folder)
        file_path = Path(getcwd())
        create_file(file_path / actions_folder / "__init__.py", "")
        write_msg(success_msg(" done!"), NewLine.after)
    except:
        write_msg(error_msg(" folder exists, skipping"), NewLine.after)

    for action_file in action_files:
        try:
            write_msg(
                info_msg("Creating %s/%s file..." % (actions_folder, action_file))
            )
            file_path = Path(getcwd())
            create_file(
                file_path / actions_folder / action_file,
                action_template(make_class_name(action_file)),
            )
            write_msg(success_msg(" done!"), NewLine.after)
        except:
            write_msg(error_msg(" file exists, skipping"), NewLine.after)

    try:
        write_msg(info_msg("Creating .pantamrc.json file..."))
        create_pantamrc_file(
            {
                "actions_folder": actions_folder,
                "entrypoint": index_file,
                "port": 5000,
                "dev_port": 5000,
            }
        )
        write_msg(success_msg(" done!"), NewLine.after)
    except:
        write_msg(error_msg(" file exists, skipping"), NewLine.after)


def run_init() -> None:
    """CLI runner for init()"""
    try:
        init()
        write_msg(
            success_msg("Your application is ready!"), NewLine.both,
        )
        write_msg(
            "Run " + success_msg("pantam serve") + " to begin...\n", NewLine.both,
        )
    except CancelError:
        write_error(error_msg("Setup cancelled..."))
    except Exception as error:
        write_error(error_msg(str(error)))


if __name__ == "__main__":
    run_init()



from sys import stderr, stdout
from enum import Enum
from colored import fg, attr

PANTAM: str = fg("yellow") + attr("bold") + "PANTAM" + attr("reset")

colour_msg = lambda msg, colour: fg(colour) + attr("bold") + msg + attr("reset")
info_msg = lambda msg: colour_msg(msg, "blue")
success_msg = lambda msg: colour_msg(msg, "green")
error_msg = lambda msg: colour_msg(msg, "red")


class NewLine(Enum):
    before = 1
    after = 2
    both = 3


def write_msg(msg: str, spacing: NewLine = None) -> None:
    """Write message to stdout"""
    prefix: str = "\n" if spacing in (NewLine.before, NewLine.both) else ""
    suffix: str = "\n" if spacing in (NewLine.after, NewLine.both) else ""
    stdout.write("%s%s%s" % (prefix, msg, suffix))


def write_error(msg: str) -> None:
    """Write message to stderr"""
    stderr.write("\n%s\n" % msg)


welcome_msg = (
    lambda: PANTAM
    + """

The microframework for microservices.

Let's build your app...

"""
)

name_index_file_msg = lambda: "What is the name of your main script?"

name_actions_folder_msg = lambda: "What is the name of your actions folder?"


def create_actions_file_msg(second_run: bool):
    """Actions File Message"""
    article = "another" if second_run else "an"
    return "Do you want to create %s action file?" % article


name_actions_file_msg = lambda: "What is the name of your actions file?"

confirm_structure_msg = (
    lambda structure: """Your application will look like this:
%s

Happy to proceed?"""
    % structure
)



index_template = lambda: """from pantam import Pantam

pantam = Pantam(debug=True)

app = pantam.build()
"""


action_template = lambda class_name: """from pantam import PlainTextResponse

class {class_name}:
    def fetch_all(self, request):
        \"\"\"Fetch all items\"\"\"
        return PlainTextResponse("Pantam: {class_name} -> fetch_all()")

    def fetch_single(self, request):
        \"\"\"Fetch single item\"\"\"
        uid = request.path_params["id"]
        return PlainTextResponse("Pantam: {class_name} -> fetch_single()")

    def create(self, request):
        \"\"\"Create an item\"\"\"
        return PlainTextResponse("Pantam: {class_name} -> create()")

    def update(self, request):
        \"\"\"Update an item\"\"\"
        uid = request.path_params["id"]
        return PlainTextResponse("Pantam: {class_name} -> update()")

    def delete(self, request):
        \"\"\"Delete single item\"\"\"
        uid = request.path_params["id"]
        return PlainTextResponse("Pantam: {class_name} -> delete()")
""".format(
    class_name=class_name
)



from typing import Any, Callable, Final, List, Literal, Optional, TypedDict, Union
from functools import reduce
from importlib import import_module
from inspect import getmembers, isfunction
from re import match, sub
from os import listdir
from starlette.applications import Starlette
from starlette.routing import Route
from .services import Logger


class Config(TypedDict):
    actions_folder: str
    actions_index: str
    debug: bool
    dev_port: int
    port: int
    reload: Optional[bool]


VERB = Literal["get", "post", "patch", "delete"]


class ActionRoute(TypedDict):
    method: str
    verb: VERB
    url: str


class ActionResource(TypedDict, total=False):
    file_name: str
    module_name: str
    class_name: str
    action_class: Callable[[], Any]
    action_obj: Any
    routes: List[ActionRoute]


class Methods(TypedDict):
    get: List[str]
    post: List[str]
    patch: List[str]
    delete: List[str]


GET_METHOD_RE = r"^(get\w*|fetch_all|fetch_single)$"
POST_METHOD_RE = r"^(set\w*|create)$"
PATCH_METHOD_RE = r"^update$"
DELETE_METHOD_RE = r"^delete$"

INDI_RES_RE = r"^(fetch_single|update|delete)$"

CUSTOM_METHOD_RE = r"^[g|s]et\w*$"


def introspect_methods(action_class: Callable[[], Any]) -> Methods:
    """Map action class methods to method dictionary"""

    def predicate(attribute: Any) -> bool:
        return isfunction(attribute)

    methods: Methods = {
        "get": list(
            filter(
                lambda x: match(GET_METHOD_RE, x),
                map(lambda x: x[0], getmembers(action_class, predicate)),
            )
        ),
        "post": list(
            filter(
                lambda x: match(POST_METHOD_RE, x),
                map(lambda x: x[0], getmembers(action_class, predicate)),
            )
        ),
        "patch": list(
            filter(
                lambda x: match(PATCH_METHOD_RE, x),
                map(lambda x: x[0], getmembers(action_class, predicate)),
            )
        ),
        "delete": list(
            filter(
                lambda x: match(DELETE_METHOD_RE, x),
                map(lambda x: x[0], getmembers(action_class, predicate)),
            )
        ),
    }
    return methods


class Pantam:
    logger: Final[Logger] = Logger()

    def __init__(
        self,
        actions_folder="actions",
        actions_index="index",
        debug=False,
        dev_port=5000,
        port=5000,
        reload=None,
    ) -> None:
        self.config: Config = {
            "actions_folder": actions_folder,
            "actions_index": actions_index,
            "debug": debug,
            "dev_port": dev_port,
            "port": port,
            "reload": reload,
        }
        self.actions: List[ActionResource] = []
        self.routes: List[Route] = []

    def get_config(self) -> Config:
        """Get Pantam app config"""
        return self.config

    def set_config(self, config: Config) -> None:
        """Set Pantam app config"""
        self.config = config

    def read_actions_folder(self) -> List[str]:
        """Read action files from actions folder"""
        config = self.get_config()
        files: List[str] = []

        def predicate(file_name: str):
            return ".py" in file_name and "__init__" not in file_name

        try:
            files.extend(filter(predicate, listdir(config["actions_folder"])))
        except:
            self.logger.error(
                "Unable to read actions folder! Check `actions_folder` config setting."
            )
        return files

    def get_actions(self) -> List[ActionResource]:
        """Get Pantam actions"""
        if len(self.actions) == 0:
            self.logger.error(
                "You have no loaded actions. Check for files in the actions folder."
            )
        return self.actions

    def log_routes(self) -> None:
        """Print Pantam routes in human readable format"""
        actions = self.get_actions()

        routes: List[ActionRoute] = []
        for action in actions:
            routes.extend(action["routes"])

        if len(routes) == 0:
            self.logger.error("No available routes!")
            return

        routes_to_log: List[str] = []
        routes_to_log.append("Available Routes:\n")

        longest_file_name = reduce(max, map(lambda z: len(z["file_name"]), actions), 0)

        longest_url = reduce(max, map(lambda z: len(z["url"]), routes), 0)

        def column_format(text: str, padding: int) -> str:
            return "{text:{padding}}".format(text=text, padding=padding)

        for action in actions:
            for route in action["routes"]:
                routes_to_log.append(
                    " -> ".join(
                        (
                            column_format(route["verb"].upper(), 6),
                            column_format(route["url"], longest_url),
                            column_format(action["file_name"], longest_file_name),
                            route["method"],
                        )
                    )
                )

        self.logger.info("\n".join(routes_to_log))

    def discover_actions(self) -> List[ActionResource]:
        """Parse methods from action files"""
        action_files = self.read_actions_folder()
        actions: List[ActionResource] = []
        for file_name in action_files:
            module_name = file_name.replace("_", "-").replace(".py", "")
            class_name = module_name.replace("-", " ").title().replace(" ", "")
            actions.append(
                {
                    "file_name": file_name,
                    "module_name": module_name,
                    "class_name": class_name,
                    "routes": [],
                }
            )
        self.actions = actions
        return actions

    def import_action_module(
        self, module_name: str, class_name: str
    ) -> Union[Callable[[], Any], None]:
        """Load an action file"""
        try:
            actions_folder = self.get_config()["actions_folder"]
            action_module: Any = import_module("%s.%s" % (actions_folder, module_name))
            action_class: Any = getattr(action_module, class_name)
            return action_class
        except:
            self.logger.error(
                "Unable to load `%s.%s` module." % (actions_folder, module_name)
            )
            return None

    def load_actions(self) -> None:
        """Import and instantiate action classes"""
        actions = self.get_actions()

        def action_loader(action: ActionResource) -> ActionResource:
            try:
                action_class = self.import_action_module(
                    action["module_name"], action["class_name"]
                )
                if action_class is not None:
                    action["action_class"] = action_class
                    action["action_obj"] = action_class()
            except:
                self.logger.error(
                    "Unable to instantiate `%s` action class." % action["class_name"]
                )
            return action

        self.actions = list(map(action_loader, actions))

    def make_url(self, module_name: str, method: str) -> str:
        """Create URL for action routes"""
        actions_index = self.get_config()["actions_index"]
        url = "/" if actions_index == module_name else "/%s/" % module_name
        is_individual_res = match(INDI_RES_RE, method)
        if is_individual_res:
            url = "%s{id}" % url
        is_custom_method = match(CUSTOM_METHOD_RE, method)
        if is_custom_method:
            slug = sub(r"^[g|s]et_", "", method).replace("_", "-").lower()
            url = "%s%s/" % (url, slug)
        return url

    def make_routes(
        self, path: str, action: Callable[[], Any] = None
    ) -> List[ActionRoute]:
        """Create action routes with method, verb, and URL"""
        if action is None:
            return []
        methods = introspect_methods(action)

        def map_to_route(verb: VERB):
            return lambda method: {
                "method": method,
                "verb": verb,
                "url": self.make_url(path, method),
            }

        get_routes = list(map(map_to_route("get"), methods["get"]))
        post_routes = list(map(map_to_route("post"), methods["post"]))
        patch_routes = list(map(map_to_route("patch"), methods["patch"]))
        delete_routes = list(map(map_to_route("delete"), methods["delete"]))
        return get_routes + post_routes + patch_routes + delete_routes

    def bind_routes(self) -> None:
        """Create starlette routes"""
        actions = self.get_actions()

        starlette_routes: List[Route] = []

        for action in actions:
            try:
                routes = self.make_routes(action["module_name"], action["action_class"])
                action["routes"] = routes

                if len(routes) == 0:
                    self.logger.error(
                        "No methods found for `%s` action." % action["module_name"]
                    )
                    continue

                def prepare_verb(verb: VERB) -> List[str]:
                    return [verb.upper()]

                for route in routes:
                    starlette_routes.append(
                        Route(
                            route["url"],
                            getattr(action["action_obj"], route["method"]),
                            methods=prepare_verb(route["verb"]),
                        )
                    )
            except:
                self.logger.error(
                    "Unable to bind `%s` action methods to route."
                    % action["module_name"]
                )

        self.routes = starlette_routes

    def get_routes(self, skip_check: bool = False) -> List[Route]:
        """Get underlying Starlette routes"""
        if len(self.routes) == 0 and skip_check is False:
            self.logger.error("No routes have been defined.")
        return self.routes

    def build(self) -> Optional[Starlette]:
        """Build Pantam application"""
        config = self.get_config()
        self.discover_actions()
        self.load_actions()
        self.bind_routes()
        routes = self.get_routes()
        if config["debug"]:
            self.log_routes()
        try:
            return Starlette(routes=routes, debug=config["debug"])
        except:
            self.logger.error("Unable to build Pantam application!")
        return None



from starlette import responses, requests
from .pantam import Pantam, introspect_methods

Request = requests.Request

JSONResponse = responses.JSONResponse
HTMLResponse = responses.HTMLResponse
PlainTextResponse = responses.PlainTextResponse
FileResponse = responses.FileResponse
RedirectResponse = responses.RedirectResponse



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pantam', 'pantam.services', 'pantam_cli', 'pantam_cli.utils']

package_data = \
{'': ['*']}

install_requires = \
['PyInquirer>=1.0.3,<2.0.0',
 'colored>=1.4.2,<2.0.0',
 'python-multipart>=0.0.5,<0.0.6',
 'shellingham>=1.3.2,<2.0.0',
 'starlette>=0.13.6,<0.14.0',
 'typer>=0.3.1,<0.4.0',
 'uvicorn>=0.11.6,<0.12.0']

entry_points = \
{'console_scripts': ['pantam = pantam_cli.cli:run']}

setup_kwargs = {
    'name': 'pantam',
    'version': '0.0.1a11',
    'description': 'The microframework for microservices',
    'long_description': '# pantam\n\n<img src="https://raw.githubusercontent.com/flmnt/pantam/master/logo/logo.png" alt="Peter the Pantam Python" width="300">\n\nPantam is an extensible, ultra lightweight, Python framework for creating RESTful microservices.\n\nFeatures include:\n\n- built on top of [Starlette](https://www.starlette.io/)\n- served using [Uvicorn](https://www.uvicorn.org/)\n- simple REST routing\n- [segregated business logic](https://en.wikipedia.org/wiki/Separation_of_concerns)\n- [convention over configuration](https://en.wikipedia.org/wiki/Convention_over_configuration) paradigm\n- expressive logging (decent error reporting!)\n- live reloading in developer mode\n\nNeed to build a Javascript microservice? [Check out Bantam](https://github.com/flmnt/bantam)\n\n## Getting started\n\nOur goal with Pantam is reduce the work bootstrapping microservices.\n\nWith Pantam you can create a basic REST API in 5 minutes or less.\n\n### Examples\n\n- [Domain segregated example](https://github.com/flmnt/pantam/tree/master/example/domain)\n- [Event based example](https://github.com/flmnt/pantam/tree/master/example/event)\n\n### Installation\n\nWe recommend using [Poetry](https://python-poetry.org/) for dependency and environment management (this documentation will use Poetry throughout) but if you prefer `pip` or `conda` then crack on!\n\nIf you haven\'t done so already, setup your python environment and Poetry project:\n\n```\n% pyenv install 3.8.2\n% pyenv local 3.8.2\n% poetry init\n```\n\n_NB: if you specify pantam as a package during the setup process, run `poetry install` and skip the next step._\n\nNow install the Pantam package:\n\n```\n% poetry add pantam\n```\n\nOnce you have installed Pantam you can initialise your app.\n\n```\n% poetry shell\n% pantam init\n```\n\nFollow the CLI instructions and then start building your microservice!\n\n### Setup\n\nPantam expects the following folder structure:\n\n```\n| main.py       // can have any name, it\'s where you run your app\n| actions        // where your domain logic sits\n|  |  index.py   // primary logic lives here (might be all you need)\n|  |  other.py   // add as many other "domains" as you like (optional)\n```\n\nIn the root level `main.py` file add the following to run Pantam:\n\n```\nfrom pantam import Pantam\n\npantam = Pantam()\n\napp = pantam.build()\n\nif __name__ == "__main__":\n    pantam.run("main", "app")\n```\n\nIn the `actions` folder create the following files.\n\n`actions/index.py`\n\n```\nclass Index:\n  def fetch_all(self):\n    pass\n\n  def fetch_single(self, uid):\n    pass\n\n  def create(self, data):\n    pass\n\n  def update(self, uid, data):\n    pass\n\n  def delete(self, uid):\n    pass\n```\n\n`actions/other.py`\n\n```\nclass Other:\n  def fetch_single(self, uid):\n    pass\n\n  def create(self, data):\n    pass\n```\n\nThe setup above will make the following routes available:\n\n```\nGET      /            // Index.fetch_all()\nGET      /:id         // Index.fetch_single()\nPOST     /            // Index.create()\nPATCH    /:id         // Index.update()\nDELETE   /:id         // Index.delete()\n\nGET      /other/:id   // Other.fetch_single()\nPOST     /other       // Other.create()\n```\n\nAnd that\'s you ready to go!\n\n### Development\n\nConfigure the application to live reload on a given port as follows:\n\n```\nfrom pantam import Pantam\n\npantam = Pantam(debug=True)\n\napp = pantam.build()\n```\n\nThen serve the app:\n\n```\n% pantam serve --dev\n```\n\nYour application will be served on http://localhost:5000\n\n_NB: Pantam looks for a `PANTAM_ENV` environment variable when serving the application and defaults to `development` if no variable is set._\n\n### Production\n\nTo serve your microservice in production use:\n\n```\n% pantam serve\n```\n\nYour application is served at http://your-host:5000\n\nYou can change the port number via the configuration options.\n\n_NB: in production you should set a `PANTAM_ENV=production` environment variable._\n\n## .pantamrc.json\n\nAfter running `pantam init` you will have a `.pantamrc.json` file in your directory with some CLI config options like this:\n\n```\n{\n  "actions_folder": "actions",\n  "entrypoint": "example.py",\n  "port": 5000,\n  "dev_port": 5000\n}\n```\n\nThe `.pantamrc.json` file provides configuration options for the CLI. You only need to change it if you change your main file (entrypoint) or rename your actions folder.\n\n## Add New Routes\n\nTo add a new action (resource) you can either create a new file in the actions folder or use the CLI to make the file for you:\n\n```\n% pantam action index.py\n```\n\nYou can add the standard methods (`fetch_all`, `fetch_single`, `create`, `update`, `delete`) to an action class which will automatically create the standard routes.\n\nIf you\'d like to create custom methods for your action class you can create custom getters like this:\n\n```\n// GET -> /custom-method/\ndef get_custom_method(self):\n  # your code here\n```\n\nAnd custom setters like this:\n\n```\n// POST -> /custom-method/\ndef set_custom_method(self, data):\n  print(data)\n  # your code here\n```\n\nPantam will ignore methods that are not "standard" methods or do not start with `get` or `set`. However if you want to _ensure_ that your method will be ignored you can prefix the method with a double underscore, like this:\n\n```\ndef __my_hidden_method(self):\n  // do something secret\n```\n\n## Creating Responses\n\nTo create a response, make use of the [Starlette response API](https://www.starlette.io/responses/), you can import all responses from starlette or import common responses from Pantam directly, including: `JSONResponse`, `HTMLResponse`, `PlainTextResponse`, `FileResponse`, `RedirectResponse`.\n\nHere are a few examples:\n\n```\nfrom pantam import PlainTextResponse\n\nclass YourClass:\n  def fetch_all():\n    return PlainTextResponse("This is fetch all!")\n```\n\n```\nfrom pantam import JSONResponse\n\nclass YourClass:\n  def fetch_all():\n    return JSONResponse([{ "content": "example" }])\n```\n\nChanging status code is also simple.\n\n```\ndef fetch_all(self):\n  return PlainTextResponse("This is fetch all!", status_code=404)\n```\n\nAdjusting headers can also be achieved.\n\n```\ndef fetch_all(self):\n  headers = {\n    "Cache-Control": "no-cache"\n  }\n  return PlainTextResponse("This is fetch all!", headers=headers)\n```\n\n## Configuration Options\n\nFor advanced configuration pass options in when instantiating Pantam.\n\n```\nfrom pantam import Pantam\n\npantam = Pantam(debug=True) # add options as below\n\napp = pantam.build()\n\nif __name__ == "__main__":\n    pantam.run("main", "app")\n```\n\nYou can set the following options:\n\n**debug**: `bool`\n\nTurns debugging mode on for Pantam (and underlying Starlette). Provides more verbose error messages. Should be off in production.\n\n`Default: False`\n\n**actions_folder**: `string`\n\nThe folder that contains your action files.\n\n`Default: "actions"`\n\n<br>\n\n**actions_index**: `string`\n\nThe primary action file in your action folder.\n\n`Default: "index"`\n\n## Debugging\n\nIf you\'re struggling to debug and issue and unsure what routes Pantam has created for you, set the `debug` option to True.\n\n```\nfrom pantam import Pantam\n\npantam = Pantam(debug=True)\n\napp = pantam.build()\n```\n\nAlso check trailing slashes in your urls, these are important.\n\nIn the example below the url `test/1` and `test/custom-method` both trigger `fetch_single()` but the url `test/custom-method/` (with the trailing slash) triggers `get_custom_method()`.\n\n```\n// actions/test.py\n\n// GET -> test/custom-method\n// GET -> test/:id\nfetch_single()\n\n// GET -> test/custom-method/\nget_custom_method()\n```\n\n## Contribution\n\nWe welcome feedback, suggestions and contributions.\n\nIf you have an idea you want to discuss please [open an issue](https://github.com/flmnt/pantam/issues/new).\n\n## Licenses\n\nFree for personal and commerical use under the [MIT License](https://github.com/flmnt/pantam/blob/master/LICENSE.md)\n',
    'author': 'Matt Davies',
    'author_email': 'matt@filament.so',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/flmnt/pantam/blob/master/README.md',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from typing import Callable, Optional
from sys import stdout
from colored import fg, attr

PANTAM: str = fg("yellow") + attr("bold") + "PANTAM: " + attr("reset")

WriteStream = Callable[[str], int]


class Logger:
    def __init__(self, write_stream: Optional[WriteStream] = None) -> None:
        if write_stream is None:
            self.write_stream = stdout.write
        else:
            self.write_stream = write_stream

    def __write(self, message: str) -> None:
        """Write message to stdout."""
        self.write_stream(str(PANTAM + message) + "\n")

    def info(self, message: str) -> None:
        """Print info message."""
        self.__write(fg("blue") + attr("bold") + message + attr("reset"))

    def success(self, message: str) -> None:
        """Print success message"""
        self.__write(fg("green") + attr("bold") + message + attr("reset"))

    def error(self, message: str) -> None:
        """Print error message"""
        self.__write(fg("red") + attr("bold") + message + attr("reset"))



#!/usr/bin/env python3

from os import getenv
import sys
from uvicorn import run
from pantam_cli.utils.filesystem import load_pantamrc_file
from pantam_cli.utils import clear


def run_serve(dev_mode: bool) -> None:
    """Serve a Pantam application"""
    clear()
    options = load_pantamrc_file()
    entrypoint = options["entrypoint"].replace(".py", "")
    listen_port = (
        options["port"]
        if getenv("PANTAM_ENV", default="development") == "production"
        else options["dev_port"]
    )
    run("%s:app" % entrypoint, port=listen_port, reload=dev_mode)


if __name__ == "__main__":
    dev = sys.argv[1] if len(sys.argv) > 1 else ""
    run_serve(dev in ("--dev", "-d"))
